if [ -x usr/bin/update-mime-database ]; then
  usr/bin/update-mime-database usr/share/mime 1> /dev/null 2> /dev/null
fi

if [ -x usr/bin/update-desktop-database ]; then
  usr/bin/update-desktop-database 1> /dev/null 2> /dev/null
fi
( cd usr/lib/gdesklets/data ; rm -rf gdesklets.png )
( cd usr/lib/gdesklets/data ; ln -sf /usr/share/pixmaps/gdesklets.png gdesklets.png )
( cd usr/bin ; rm -rf gdesklets )
( cd usr/bin ; ln -sf /usr/lib/gdesklets/gdesklets gdesklets )
