class __Null:

    """Null object that does __nothing__"""

    def __init__(self, *args, **kwargs): pass
    def __call__(self, *args, **kwargs): return self
    def __repr__(self): return 'Null'
    def __str__(self): return 'Null'
    def __nonzero__(self): return 0

    def __getattr__(self, name): return self
    def __setattr__(self, name, value): return self
    def __delattr__(self, name): return self


# Defines Null as a builtin
# we don't need more than one instance
import __builtin__
__builtin__.Null = __Null()
