from ConfigFloat import ConfigFloat


class ConfigInteger(ConfigFloat):

    def __init__(self, name, getter, setter, caller):

        ConfigFloat.__init__(self, name, getter, setter, caller,
                             int_only = True)
