TIPS = (
    _("You can find lots of desktop applets for use\n"
      "with gDesklets at\n"
      "\n"
      "http://gdesklets.gnomedesktop.org"),

    _("You can install new desktop applets easily by\n"
      "dragging the download link for the package\n"
      "from your web browser into the gDesklets Shell."),

    _("Click the right mouse button on a desktop\n"
      "applet or press F10 to open the popup menu."),

    _("You can move the applets around the screen\n"
      "by holding down the middle mouse button.\n"
      "\n"
      "If your mouse has only two buttons,\n"
      "hold down both of them simultaneously\n"
      "in order to simulate the middle button."),

    _("You can have different profiles of\n"
      "desktop applets and switch between them.\n"
      "\n"
      "Select \"Profiles->New profile...\" from\n"
      "the menu bar to create new profiles.\n"
      "\n"
      "Empty profiles get discarded automatically."),

    _("The search bar accepts complex search\n"
      "expressions for finding your applets."),

    _("Search expressions may contain wildcards:\n"
      "\n"
      " ? matches any one character\n"
      " * matches zero or more characters")
    )
