COMMAND_OPEN_DISPLAY = "open display"
COMMAND_OPEN_DISPLAY_WITH_ID = "open display with id"
COMMAND_OPEN_PLUG = "open plug"
COMMAND_CLOSE_DISPLAY = "close display"
COMMAND_SHUTDOWN = "shutdown"
COMMAND_DISPLAYS = "displays"
COMMAND_DISPLAY_LIST = "display list"
COMMAND_GEOMETRY = "geometry"
COMMAND_SET_REMOVE_COMMAND = "set remove command"
COMMAND_VERSION = "version"
COMMAND_ABOUT = "about"
COMMAND_CONFIGURE = "configure"
